@@ -1,5 +1,5 @@
 // Generic output functions.
-use crate::color::RgbColor;
+use crate::color::{self, RgbColor};
 use crate::common::{self, wcs2string_appending};
 use crate::curses::{self, tparm1, Term};
 use crate::env::EnvVar;
@@ -456,6 +456,35 @@ impl Outputter {
     }
 }
 
+pub struct BufferedOuputter<'a>(&'a mut Outputter);
+
+impl<'a> BufferedOuputter<'a> {
+    pub fn new(outputter: &'a mut Outputter) -> Self {
+        outputter.begin_buffering();
+        Self(outputter)
+    }
+}
+
+impl<'a> Drop for BufferedOuputter<'a> {
+    fn drop(&mut self) {
+        self.0.end_buffering();
+    }
+}
+
+impl<'a> Write for BufferedOuputter<'a> {
+    fn write(&mut self, buf: &[u8]) -> Result<usize> {
+        self.0
+            .write(buf)
+            .expect("Writing to in-memory buffer should never fail");
+        Ok(buf.len())
+    }
+
+    fn flush(&mut self) -> Result<()> {
+        self.0.flush().unwrap();
+        Ok(())
+    }
+}
+
 /// Given a list of RgbColor, pick the "best" one, as determined by the color support. Returns
 /// RgbColor::NONE if empty.
 pub fn best_color(candidates: &[RgbColor], support: ColorSupport) -> RgbColor {
@@ -493,6 +522,14 @@ pub fn best_color(candidates: &[RgbColor], support: ColorSupport) -> RgbColor {
 ///       In particular, the argument parsing still isn't fully capable.
 #[allow(clippy::collapsible_else_if)]
 pub fn parse_color(var: &EnvVar, is_background: bool) -> RgbColor {
+    let mut result = parse_color_maybe_none(var, is_background);
+    if result.is_none() {
+        result.typ = color::Type::Normal;
+    }
+    result
+}
+
+pub fn parse_color_maybe_none(var: &EnvVar, is_background: bool) -> RgbColor {
     let mut is_bold = false;
     let mut is_underline = false;
     let mut is_italics = false;
@@ -551,9 +588,6 @@ pub fn parse_color(var: &EnvVar, is_background: bool) -> RgbColor {
     }
 
     let mut result = best_color(&candidates, get_color_support());
-    if result.is_none() {
-        result = RgbColor::NORMAL;
-    }
     result.set_bold(is_bold);
     result.set_underline(is_underline);
     result.set_italics(is_italics);
