@@ -1,7 +1,12 @@
 #!/usr/bin/env python3
 from pexpect_helper import SpawnedProc
 import os
+import sys
 import signal
+import platform
+
+if "CI" in os.environ and platform.system() in ["Darwin", "FreeBSD"]:
+    sys.exit(127)
 
 sp = SpawnedProc()
 send, sendline, sleep, expect_prompt = sp.send, sp.sendline, sp.sleep, sp.expect_prompt
@@ -11,30 +16,33 @@ send("set -g fish_key_bindings fish_vi_key_bindings\r")
 expect_prompt()
 
 send("echo ready to go\r")
-expect_prompt("\r\nready to go\r\n")
+expect_prompt(f"\r\n.*ready to go\r\n")
 send(
     "function add_change --on-variable fish_bind_mode ; set -g MODE_CHANGES $MODE_CHANGES $fish_bind_mode ; end\r"
 )
 expect_prompt()
 
+sendline("echo 'Catch' 'up'")
+expect_prompt("Catch up")
+
 # normal mode
 send("\033")
-sleep(0.050)
+sleep(10 if "CI" in os.environ else 1)
 
 # insert mode
 send("i")
-sleep(0.050)
+sleep(10 if "CI" in os.environ else 1)
 
 # back to normal mode
 send("\033")
-sleep(0.050)
+sleep(10 if "CI" in os.environ else 1)
 
 # insert mode again
 send("i")
-sleep(0.050)
+sleep(10 if "CI" in os.environ else 1)
 
 send("echo mode changes: $MODE_CHANGES\r")
-expect_prompt("\r\nmode changes: default insert default insert\r\n")
+expect_prompt("\r\n.*mode changes: default insert default insert\r\n")
 
 # Regression test for #8125.
 # Control-C should return us to insert mode.
@@ -62,4 +70,4 @@ sleep(timeout)
 
 # We should be back in insert mode now.
 send("echo mode changes: $MODE_CHANGES\r")
-expect_prompt("\r\nmode changes: default insert\r\n")
+expect_prompt("\r\n.*mode changes: default insert\r\n")
